"use strict";

import {
  BASE_URL,
  QUERY_PARAMS_COMBINED,
} from "./src_modules/baseUrlPlusQueryParams.mjs";

import { getMetadataPlusObservationsFromSingleOrMultipleDatastreams } from "./src_modules/fetchData.mjs";

import {
  formatDatastreamMetadataForChart,
  extractPropertiesFromFormattedDatastreamMetadata,
} from "./src_modules/fetchedDataProcessing.mjs";

import {
  formatSensorThingsApiResponseForLineOrColumnChart,
  drawLineChartHighcharts,
} from "./src_modules/chartLine.mjs";

import {
  formatSensorThingsApiResponseForHeatMap,
  drawHeatMapHighcharts,
} from "./src_modules/chartHeatmap.mjs";

import {
  showLoadingSpinner,
  hideLoadingSpinner,
} from "./src_modules/loadingIndicator.mjs";

import { vanillaSelectBox } from "./thirdparty/vanillaSelectBox.mjs";

/**
 * Use the `vanillaDropDown` library to style the first level drop down list
 *
 * @returns {undefined}
 */
const styleLevelOneDropDown = function () {
  // Create our dropdown list using `vanillaSelectBox`; supports the selection of multiple options
  new vanillaSelectBox("#drop-down--bldg-data-point", {
    "disableSelectAll": true,
    "maxSelect": 5,
    "placeHolder": "--Select--",
    "search": false,
  });
};

/**
 * Use the `vanillaDropDown` library to style the second level drop down list
 *
 * @returns {undefined}
 */
const styleLevelTwoDropDown = function () {
  // Create our dropdown list using `vanillaSelectBox`
  new vanillaSelectBox("#drop-down--aggregation-type", {
    "placeHolder": "--Select--",
  });
};

/**
 * Use the `vanillaDropDown` library to style the third level drop down list
 *
 * @returns {undefined}
 */
const styleLevelThreeDropDown = function () {
  // Create our dropdown list using `vanillaSelectBox`
  new vanillaSelectBox("#drop-down--sampling-rate", {
    "placeHolder": "--Select--",
  });
};

/**
 * Use the `vanillaDropDown` library to style the fourth level drop down list
 *
 * @returns {undefined}
 */
const styleLevelFourDropDown = function () {
  // Create our dropdown list using `vanillaSelectBox`
  new vanillaSelectBox("#drop-down--chart-type", {
    "placeHolder": "--Select--",
  });
};

/**
 * Callback function that wraps the logic of populating the linked drop down lists.
 * Will run on `DOMContentLoaded` event
 *
 * @returns {undefined}
 */
const afterDocumentLoads = function () {
  styleLevelOneDropDown();
  styleLevelTwoDropDown();
  styleLevelThreeDropDown();
  styleLevelFourDropDown();
};

/**
 * Get the values from the currently selected options in the linked drop dpwn lists
 * @returns {Array} An array containing the values of the selected options
 */
const getSelectedOptionsFromDropDownLists = function () {
  const selectedBuilding = document.querySelector("#drop-down--bldg").value;
  const selectedSensor = document.querySelector("#drop-down--sensor").value;
  const selectedSamplingRate = document.querySelector(
    "#drop-down--sampling-rate"
  ).value;

  if (
    selectedBuilding === "--Select--" ||
    selectedSensor === "" ||
    selectedSamplingRate === ""
  )
    return;

  return [selectedBuilding, selectedSensor, selectedSamplingRate];
};

/**
 * Get the abbreviated form of building IDs, phenomenon names and sensor sampling rates
 * @param {String} buildingFullForm A string representation of the full form of a building ID
 * @param {String} phenomenonFullForm A string representation of the full form of a phenomenon name
 * @param {String} samplingRateFullForm A string representation of the full form of a sensor's sampling rate
 * @returns {Array} An array of abbreviated strings
 */
const getBuildingSensorSamplingRateAbbreviation = function (
  buildingFullForm,
  phenomenonFullForm,
  samplingRateFullForm
) {
  const fullFormToAbbreviationMapping = {
    buildings: {
      "Bau 101": "101",
      "Bau 102": "102",
      "Bau 107": "107",
      "Bau 112": "112, 118",
      "Bau 125": "125",
      "Bau 225": "225",
    },
    phenomenon: {
      Vorlauftemperatur: "vl",
      Rücklauftemperatur: "rl",
      Durchfluss: "flow",
      Leistung: "power",
      Energie: "energy",
      Energie_VERBR: "energy_verb",
    },
    samplingRate: {
      "15 min": "15min",
      "60 min": "60min",
    },
  };

  const buildingAbbrev =
    fullFormToAbbreviationMapping["buildings"]?.[buildingFullForm];

  const phenomenonAbbrev =
    fullFormToAbbreviationMapping["phenomenon"]?.[phenomenonFullForm];

  const samplingRateAbbrev =
    fullFormToAbbreviationMapping["samplingRate"]?.[samplingRateFullForm];

  return [buildingAbbrev, phenomenonAbbrev, samplingRateAbbrev];
};

/**
 * Callback function for chart selection using drop down list
 * @returns {undefined}
 */
const selectChartTypeFromDropDown = async function () {
  try {
    const selectedOptions = getSelectedOptionsFromDropDownLists();

    if (selectedOptions === undefined) return;

    const selectedOptionsAbbreviationsArr =
      getBuildingSensorSamplingRateAbbreviation(...selectedOptions);

    const selectedChartType = document.querySelector(
      "#drop-down--chart-type"
    ).value;

    if (selectedChartType === "--Select--") return;

    // Display the loading indicator
    showLoadingSpinner();

    // The `getMetadataPlusObservationsFromSingleOrMultipleDatastreams` function expects a nested array structure
    const abbreviationsNestedArr = [selectedOptionsAbbreviationsArr];

    const observationsPlusMetadata =
      await getMetadataPlusObservationsFromSingleOrMultipleDatastreams(
        BASE_URL,
        QUERY_PARAMS_COMBINED,
        abbreviationsNestedArr
      );

    // Extract the combined arrays for observations and metadata
    const [observationsNestedArr, metadataNestedArr] = observationsPlusMetadata;

    // Create formatted array(s) for observations - line chart
    const formattedObsLineChartNestedArr = observationsNestedArr.map(
      (observationsArr) =>
        formatSensorThingsApiResponseForLineOrColumnChart(observationsArr)
    );

    // Create formatted array(s) for observations - heatmap
    const formattedObsHeatMapNestedArr = observationsNestedArr.map(
      (observationsArr) =>
        formatSensorThingsApiResponseForHeatMap(observationsArr)
    );

    // Create formatted array(s) for metadata - same for both chart types
    const formattedMetadataArr = metadataNestedArr.map((metadataObj) =>
      formatDatastreamMetadataForChart(metadataObj)
    );

    // Extract the formatted metadata properties
    const extractedFormattedDatastreamProperties =
      extractPropertiesFromFormattedDatastreamMetadata(
        formattedMetadataArr,
        false
      );

    if (selectedChartType === "Line") {
      drawLineChartHighcharts(
        formattedObsLineChartNestedArr,
        extractedFormattedDatastreamProperties
      );
    } else if (selectedChartType === "Heatmap") {
      // First need to extract the formatted observations from the nested array
      // Heatmap only needs one set of formatted observation values
      drawHeatMapHighcharts(
        ...formattedObsHeatMapNestedArr,
        extractedFormattedDatastreamProperties
      );
    }
  } catch (err) {
    console.error(err);
  } finally {
    // Hide the loading indicator
    hideLoadingSpinner();
  }
};

document.addEventListener("DOMContentLoaded", afterDocumentLoads);

document
  .querySelector("#drop-down--chart-type")
  .addEventListener("change", selectChartTypeFromDropDown);
